@@ -1,19 +1,20 @@
 WFO.match <- function(
     spec.data=NULL, WFO.file=NULL, WFO.data=NULL,
-    spec.name="spec.name", Genus="Genus", Species="Species", 
+    no.dates=TRUE,
+    spec.name="spec.name", Genus="Genus", Species="Species",
     Infraspecific.rank="Infraspecific.rank", Infraspecific="Infraspecific",
     Authorship="Authorship", First.dist=FALSE,
     acceptedNameUsageID.match=TRUE,
-    Fuzzy=0.1, Fuzzy.force=FALSE, Fuzzy.max=250, Fuzzy.min=TRUE, Fuzzy.shortest=FALSE, Fuzzy.within=FALSE, 
+    Fuzzy=0.1, Fuzzy.force=FALSE, Fuzzy.max=250, Fuzzy.min=TRUE, Fuzzy.shortest=FALSE, Fuzzy.within=FALSE,
     Fuzzy.two=TRUE, Fuzzy.one=TRUE,
     squish=TRUE,
     spec.name.tolower=FALSE, spec.name.nonumber=TRUE, spec.name.nobrackets=TRUE,
-    exclude.infraspecific=FALSE, 
+    exclude.infraspecific=FALSE,
     infraspecific.excluded=c("cultivar.", "f.", "sect.", "subf.", "subg.", "subsp.", "subvar.", "var", "var."),
-    spec.name.sub=TRUE, 
-    sub.pattern=c(" sp[.] A", " sp[.] B", " sp[.] C", " sp[.]", " spp[.]", " pl[.]", " indet[.]", " ind[.]", " gen[.]", " g[.]", " fam[.]", 
-        " nov[.]", " prox[.]", " cf[.]", " aff[.]", " s[.]s[.]", " s[.]l[.]", " p[.]p[.]", " p[.] p[.]", "[?]", " inc[.]", " stet[.]", 
-        "Ca[.]", "nom[.] cons[.]", "nom[.] dub[.]", " nom[.] err[.]", " nom[.] illeg[.]", " nom[.] inval[.]", " nom[.] nov[.]", 
+    spec.name.sub=TRUE,
+    sub.pattern=c(" sp[.] A", " sp[.] B", " sp[.] C", " sp[.]", " spp[.]", " pl[.]", " indet[.]", " ind[.]", " gen[.]", " g[.]", " fam[.]",
+        " nov[.]", " prox[.]", " cf[.]", " aff[.]", " s[.]s[.]", " s[.]l[.]", " p[.]p[.]", " p[.] p[.]", "[?]", " inc[.]", " stet[.]",
+        "Ca[.]", "nom[.] cons[.]", "nom[.] dub[.]", " nom[.] err[.]", " nom[.] illeg[.]", " nom[.] inval[.]", " nom[.] nov[.]",
         " nom[.] nud[.]", " nom[.] obl[.]", " nom[.] prot[.]", " nom[.] rej[.]", " nom[.] supp[.]", " sensu auct[.]"),
     verbose=TRUE, counter=1000
 )
@@ -26,14 +27,22 @@ WFO.match <- function(
     if (is.null(WFO.data) == TRUE) {
         message(paste("Reading WFO data"))
         WFO.data <- data.table::fread(WFO.file, encoding="UTF-8")
+    }else{
+        WFO.data <- data.table::data.table(WFO.data)
     }
 
 ## avoid problems with dates (error reported by Lauri Vesa)
-    WFO.data$created <- as.Date(as.character(WFO.data$created))
-    WFO.data$modified <- as.Date(as.character(WFO.data$modified))
+##    WFO.data$created <- as.Date(as.character(WFO.data$created))
+##    WFO.data$modified <- as.Date(as.character(WFO.data$modified))
 
-    WFO.data$created[is.na(WFO.data$created)] <- as.Date("1000-01-01")
-    WFO.data$modified[is.na(WFO.data$modified)] <- as.Date("1000-01-01")
+##   WFO.data$created[is.na(WFO.data$created)] <- as.Date("1000-01-01")
+##   WFO.data$modified[is.na(WFO.data$modified)] <- as.Date("1000-01-01")
+
+## from version 1.9 removed columns of created and modified, as handling these as dates
+## takes quite some time (reported by Pavel Pipek in August 2021)
+
+    if ("created" %in% names(WFO.data)) {data.table::set(WFO.data, j="created", value=NULL)}
+    if ("modified" %in% names(WFO.data)) {data.table::set(WFO.data, j="modified", value=NULL)}
 
     WFO.names <- names(WFO.data)
     WFO.names <- c(WFO.names, "Hybrid")
@@ -52,7 +61,7 @@ WFO.match <- function(
         }
     }
     if (spec.name %in% names(spec.data)) {
-        spec.data[, spec.name] <- as.character(spec.data[, spec.name])    
+        spec.data[, spec.name] <- as.character(spec.data[, spec.name])
         for (i in 1:nrow(spec.data)) {
             if (is.na(spec.data[i, spec.name]) == TRUE) {spec.data[i, spec.name] <- ""}
         }
@@ -66,7 +75,7 @@ WFO.match <- function(
                     if (spec.data[i, spec.name.ORIG] != spec.data[i, spec.name]) {spec.data[i, "Squished"] <- as.logical(1)}
                 }
             }
-        }        
+        }
         if (spec.name.tolower == TRUE) {
             if (squish == FALSE) {
                 spec.name.ORIG <- paste(spec.name, ".ORIG", sep="")
@@ -74,7 +83,7 @@ WFO.match <- function(
             }
             spec.data[, spec.name] <- tolower(spec.data[, spec.name.ORIG])
             for (i in 1:nrow(spec.data)) {
-                substr(spec.data[i, spec.name], start=1, stop=1) <- toupper(substr(spec.data[i, spec.name], start=1, stop=1)) 
+                substr(spec.data[i, spec.name], start=1, stop=1) <- toupper(substr(spec.data[i, spec.name], start=1, stop=1))
             }
         }
         if (spec.name.sub == TRUE) {
@@ -93,7 +102,7 @@ WFO.match <- function(
             }
             spec.data$Brackets.detected <- rep(as.logical(0), nrow(spec.data))
             for (i in 1:nrow(spec.data)) {
-                species.string <- spec.data[i, spec.name]        
+                species.string <- spec.data[i, spec.name]
                 if (grepl(pattern="[(]", x=species.string) == TRUE) {
                     spec.data[i, "Brackets.detected"] <- as.logical(1)
                     brack.place <- as.numeric(unlist(gregexpr(pattern="[(]", text=species.string)))[1]
@@ -131,9 +140,9 @@ WFO.match <- function(
                 species.string <- spec.data[i, spec.name]
                 if (grepl(" x ", species.string) == TRUE) {
                     species.new.string <- gsub(pattern=" x ", replacement=paste(" ", intToUtf8(215), sep=""), x=species.string)
-                    spec.data[i, spec.name] <- species.new.string            
+                    spec.data[i, spec.name] <- species.new.string
                 }
-            }  
+            }
         }
     }
 
@@ -152,7 +161,7 @@ WFO.match <- function(
 #    if (acceptedNameUsageID.match == TRUE) {spec.data$Auth.dist <- rep(Inf, nrow(spec.data))}
     if (Authorship %in% names(spec.data)) {spec.data$Auth.dist <- rep(Inf, nrow(spec.data))}
     if (First.dist == TRUE) {spec.data$First.dist <- rep(Inf, nrow(spec.data))}
-    
+
     spec.data$OriSeq <- c(1: nrow(spec.data))
     spec.data$Subseq <- rep(1, nrow(spec.data))
     init.column <- ncol(spec.data)
@@ -172,8 +181,8 @@ WFO.match <- function(
     WFO.empty <- WFO.data[1, ]
     for (i in 1:ncol(WFO.empty)) {WFO.empty[, i] <- ""}
 ## avoid problems with dates (error reported by Lauri Vesa)
-    WFO.empty$created <- as.Date("1000-01-02")
-    WFO.empty$modified <- as.Date("1000-01-02")
+##    WFO.empty$created <- as.Date("1000-01-02")
+##    WFO.empty$modified <- as.Date("1000-01-02")
 
     for (i in 1:nrow(spec.data)) {
 
@@ -183,7 +192,7 @@ WFO.match <- function(
 
         if (spec.name %in% names(spec.data) && nchar(spec.data[i, spec.name]) > 0) {
             WFO.match <- WFO.data[WFO.data$scientificName==spec.data[i, spec.name],]
-            
+
             if (nrow(WFO.match) == 0) {
                  if (spec.data[i, spec.name] == "Compositae") {WFO.match <- WFO.data[WFO.data$scientificName=="Asteraceae", ]}
                  if (spec.data[i, spec.name] == "Leguminosae") {WFO.match <- WFO.data[WFO.data$scientificName=="Fabaceae", ]}
@@ -191,10 +200,10 @@ WFO.match <- function(
                  if (spec.data[i, spec.name] == "Palmae") {WFO.match <- WFO.data[WFO.data$scientificName=="Arecaceae", ]}
                  if (spec.data[i, spec.name] == "Cruciferae") {WFO.match <- WFO.data[WFO.data$scientificName=="Brassicaceae", ]}
                  if (spec.data[i, spec.name] == "Guttiferae") {WFO.match <- WFO.data[WFO.data$scientificName=="Clusiaceae", ]}
-                 if (spec.data[i, spec.name] == "Labiatae") {WFO.match <- WFO.data[WFO.data$scientificName=="Lamiaceae", ]}      
-                 if (spec.data[i, spec.name] == "Gramineae") {WFO.match <- WFO.data[WFO.data$scientificName=="Poaceae", ]}            
+                 if (spec.data[i, spec.name] == "Labiatae") {WFO.match <- WFO.data[WFO.data$scientificName=="Lamiaceae", ]}
+                 if (spec.data[i, spec.name] == "Gramineae") {WFO.match <- WFO.data[WFO.data$scientificName=="Poaceae", ]}
             }
-            
+
             if ((nrow(WFO.match) == 0 && Fuzzy > 0) || Fuzzy.force == TRUE) {
                 specFuzzy <- agrep(spec.data[i, spec.name], x=WFO.data$scientificName, value=T, max.distance=Fuzzy)
 
@@ -204,7 +213,7 @@ WFO.match <- function(
                     if (length(species.terms) > 2) {
                         species.string2 <- paste(species.terms[1], " ", species.terms[2], sep="")
                         specFuzzy <- agrep(species.string2, x=WFO.data$scientificName, value=T, max.distance=Fuzzy)
-                        spec.data[i, "Fuzzy.two"] <- as.logical(1)                     
+                        spec.data[i, "Fuzzy.two"] <- as.logical(1)
                         if (verbose == TRUE  && length(specFuzzy) > 0) {message(paste("Fuzzy matches for ", spec.data[i, spec.name], " were only found for first 2 terms", sep=""))}
                     }
                 }
@@ -216,7 +225,7 @@ WFO.match <- function(
                         spec.data[i, "Fuzzy.one"] <- as.logical(1)
                         WFO.match <- WFO.data[WFO.data$scientificName==species.string2,]
                         if ((nrow(WFO.match) == 0 && Fuzzy > 0) || Fuzzy.force == TRUE) {
-                            specFuzzy <- agrep(species.string2, x=WFO.data$scientificName, value=T, max.distance=Fuzzy)                         
+                            specFuzzy <- agrep(species.string2, x=WFO.data$scientificName, value=T, max.distance=Fuzzy)
                             if (verbose == TRUE  && length(specFuzzy) > 0) {message(paste("Fuzzy matches for ", spec.data[i, spec.name], " were only found for first term", sep=""))}
                         }else{
                             specFuzzy <- NULL
@@ -227,7 +236,7 @@ WFO.match <- function(
                 }
 
                 if (length(specFuzzy) > Fuzzy.max) {
-                    spec.data[i, "Fuzzy.toomany"] <- length(specFuzzy)                    
+                    spec.data[i, "Fuzzy.toomany"] <- length(specFuzzy)
                     if (verbose == TRUE) {message(paste("Too many (", length(specFuzzy), ") fuzzy matches for ", spec.data[i, spec.name], ", including ", specFuzzy[1], sep=""))}
                     specFuzzy <- NULL
                 }
@@ -237,7 +246,7 @@ WFO.match <- function(
                     fuzzy.matches <- TRUE
                     specFuzzy <- unique(specFuzzy)
                     if (verbose == TRUE) {message(paste("Fuzzy matches for ", spec.data[i, spec.name], "were: ", paste(specFuzzy, collapse=", ")))}
-                    
+
                     if (spec.data[i, "Fuzzy.two"] == TRUE && spec.data[i, "Fuzzy.one"] == FALSE) {
                         specFuzzy.2 <- NULL
                         for (j in 1:length(specFuzzy)) {
@@ -249,7 +258,7 @@ WFO.match <- function(
                             specFuzzy <- specFuzzy.2
                         }
                     }
-                    
+
                     if (spec.data[i, "Fuzzy.one"] == TRUE) {
                         specFuzzy.2 <- NULL
                         for (j in 1:length(specFuzzy)) {
@@ -259,9 +268,9 @@ WFO.match <- function(
                         if (length(specFuzzy.2) > 0) {
                             if (verbose == TRUE) {message(paste("With Fuzzy.one, reduced matches to those of 1 word only"))}
                             specFuzzy <- specFuzzy.2
-                        }            
+                        }
                     }
-                                        
+
                     if (Fuzzy.within == TRUE) {
                         Fuzzy.shortest <- Fuzzy.min <- FALSE
                         within.matches <- grepl(spec.data[i, spec.name], x=specFuzzy)
@@ -302,10 +311,10 @@ WFO.match <- function(
 # Only match genus and species separately if specName was not given
         }else{
             if (Infraspecific.rank %in% names(spec.data)) {
-                WFO.match <- WFO.data[WFO.data$genus==spec.data[i, Genus] & WFO.data$specificEpithet==spec.data[i, Species] 
+                WFO.match <- WFO.data[WFO.data$genus==spec.data[i, Genus] & WFO.data$specificEpithet==spec.data[i, Species]
                     & WFO.data$verbatimTaxonRank==spec.data[i, Infraspecific.rank] & WFO.data$infraspecificEpithet==spec.data[i, Infraspecific], ]
             }else if (Infraspecific %in% names(spec.data)) {
-                WFO.match <- WFO.data[WFO.data$genus==spec.data[i, Genus] & WFO.data$specificEpithet==spec.data[i, Species] 
+                WFO.match <- WFO.data[WFO.data$genus==spec.data[i, Genus] & WFO.data$specificEpithet==spec.data[i, Species]
                     & WFO.data$infraspecificEpithet==spec.data[i, Infraspecific], ]
             }else if (Species %in% names(spec.data)){
                 WFO.match <- WFO.data[WFO.data$genus==spec.data[i, Genus] & WFO.data$specificEpithet==spec.data[i, Species], ]
@@ -318,7 +327,7 @@ WFO.match <- function(
             spec.data[i, "Unique"] <- as.logical(0)
             WFO.match2 <- cbind(spec.data[rep(i, nrow(WFO.match)), ], WFO.match)
             WFO.match2$Subseq <- c(1:nrow(WFO.match))
-        }else if (nrow(WFO.match) == 1) { 
+        }else if (nrow(WFO.match) == 1) {
             WFO.match2 <- cbind(spec.data[i, ], WFO.match)
         }else{
             spec.data[i, "Matched"] <- as.logical(0)
@@ -329,7 +338,7 @@ WFO.match <- function(
         if (fuzzy.matches == TRUE) {
             for (j in 1:nrow(WFO.match2)) {
                 WFO.match2[j, "Fuzzy.dist"] <- as.numeric(utils::adist(WFO.match2[j, "scientificName"], y=spec.data[i, spec.name]))
-                
+
                 if (First.dist == TRUE) {
                     genus.input <- spec.data[i, spec.name]
                     genus.input2 <- unlist(strsplit(genus.input, split= " "))[1]
@@ -337,9 +346,9 @@ WFO.match <- function(
                     genus.match2 <- unlist(strsplit(genus.match, split= " "))[1]
                     Fuzzy.dist1 <- as.numeric(utils::adist(genus.input2, y=genus.match2))
                     if (is.na(Fuzzy.dist1) == TRUE) {Fuzzy.dist1 <- Inf}
-                    WFO.match2[j, "First.dist"] <- Fuzzy.dist1   
+                    WFO.match2[j, "First.dist"] <- Fuzzy.dist1
                 }
-               
+
             }
         }
 
@@ -422,7 +431,7 @@ WFO.match <- function(
             Fuzzy.dist <- as.numeric(utils::adist(WFO.out[i, Authorship], y=WFO.out[i, "scientificNameAuthorship"]))
             if (is.na(Fuzzy.dist) == TRUE) {Fuzzy.dist <- Inf}
             WFO.out[i, "Auth.dist"] <- Fuzzy.dist
-        }        
+        }
     }
 
 #    if (is.null(filename) == FALSE) {utils::write.table(WFO.out, file=filename, quote=F, sep="\t", row.names=F, append=append)}
